@@ -18,75 +18,13 @@ describe('Make a request', () => {
   beforeEach(() => clock = lolex.install());
   afterEach(() => clock.uninstall());
 
-  describe('with callback', () => {
-    it('Gives contents of page', (done) => {
-      const scope = nock('http://website.com')
-        .get('/path')
-        .replyWithFile(200, __filename);
-      miniget('http://website.com/path', (err, res, body) => {
-        assert.ifError(err);
-        scope.done();
-        assert.equal(res.statusCode, 200);
-        assert.ok(body.length > 100);
-        done();
-      });
-    });
-
-    describe('with options', () => {
-      it('Makes request with options', (done) => {
-        const scope = nock('http://website.com', {
-          reqheaders: { 'User-Agent': 'miniget' },
-        })
-          .get('/path')
-          .replyWithFile(200, __filename);
-        miniget('http://website.com/path', {
-          headers: { 'User-Agent': 'miniget' },
-        }, (err, res, body) => {
-          assert.ifError(err);
-          scope.done();
-          assert.equal(res.statusCode, 200);
-          assert.ok(body.length > 100);
-          done();
-        });
-      });
-    });
-
-    describe('that errors', () => {
-      it('Calls callback with error', (done) => {
-        const scope = nock('https://mysite.com')
-          .get('/path')
-          .replyWithError('ENOTFOUND');
-        miniget('https://mysite.com/path', { maxRetries: 0 }, (err) => {
-          assert.ok(err);
-          assert.equal(err.message, 'ENOTFOUND');
-          scope.done();
-          done();
-        });
-      });
-    });
-
-    describe('with bad path', () => {
-      it('Calls callback with error', (done) => {
-        const scope = nock('https://mysite.com')
-          .get('/badpath')
-          .reply(404, 'not exists');
-        miniget('https://mysite.com/badpath', (err) => {
-          assert.ok(err);
-          scope.done();
-          done();
-        });
-      });
-    });
-  });
-
-  describe('with a promise', () => {
-    it('Gives contents of page', async () => {
+  describe('with `.text()`', () => {
+    it('Gives entire contents of page', async () => {
       const scope = nock('http://webby.com')
         .get('/pathos')
         .replyWithFile(200, __filename);
-      let [res, body] = await miniget.promise('http://webby.com/pathos');
+      let body = await miniget('http://webby.com/pathos').text();
       scope.done();
-      assert.equal(res.statusCode, 200);
       assert.ok(body.length > 100);
     });
 
@@ -96,7 +34,7 @@ describe('Make a request', () => {
           .get('/one/two/three')
           .replyWithError('NONONO');
         await assert.rejects(
-          miniget.promise('http://something.com/one/two/three', { maxRetries: 0 }),
+          miniget('http://something.com/one/two/three', { maxRetries: 0 }).text(),
           null, 'NONONO'
         );
         scope.done();
@@ -104,6 +42,53 @@ describe('Make a request', () => {
     });
   });
 
+  describe('to a working site', () => {
+    it('Returns a stream', (done) => {
+      const scope = nock('http://website.com')
+        .get('/path')
+        .replyWithFile(200, __filename);
+      const stream = miniget('http://website.com/path');
+      stream.on('error', done);
+      stream.on('response', (res) => {
+        res.on('error', done);
+        res.on('end', () => {
+          scope.done();
+          done();
+        });
+        res.resume();
+      });
+    });
+  });
+
+  describe('with options', () => {
+    it('Makes request with options', async () => {
+      const scope = nock('http://website.com', {
+        reqheaders: { 'User-Agent': 'miniget' },
+      })
+        .get('/path')
+        .replyWithFile(200, __filename);
+      let body = await miniget('http://website.com/path', {
+        headers: { 'User-Agent': 'miniget' },
+      }).text();
+      scope.done();
+      assert.ok(body.length > 100);
+    });
+  });
+
+  describe('with bad path', () => {
+    it('Emits error', (done) => {
+      const scope = nock('https://mysite.com')
+        .get('/badpath')
+        .reply(404, 'not exists');
+      let stream = miniget('https://mysite.com/badpath');
+      stream.on('error', (err) => {
+        scope.done();
+        assert.ok(err);
+        done();
+      });
+    });
+  });
+
   describe('that errors', () => {
     it('Emits error event', (done) => {
       const scope = nock('https://mysite.com')
@@ -155,40 +140,22 @@ describe('Make a request', () => {
     });
   });
 
-  describe('without callback', () => {
-    it('Returns a stream', (done) => {
-      const scope = nock('http://website.com')
-        .get('/path')
-        .replyWithFile(200, __filename);
-      const stream = miniget('http://website.com/path');
-      stream.on('error', done);
-      stream.on('response', (res) => {
-        res.on('error', done);
-        res.on('end', () => {
-          scope.done();
-          done();
-        });
-        res.resume();
+  describe('with an incorrect URL', () => {
+    it('Emits error', (done) => {
+      miniget('file:///path/to/file/').on('error', (err) => {
+        assert.ok(err);
+        assert.equal(err.message, 'Invalid URL: file:///path/to/file/');
+        done();
       });
     });
   });
 
-  describe('with an incorrect URL', () => {
-    describe('with callback', () => {
-      it('Called with error', (done) => {
-        miniget('file:///Users/roly/', (err) => {
-          assert.ok(err);
-          done();
-        });
-      });
-    });
-
-    describe('without callback', () => {
-      it('Throws error', (done) => {
-        miniget('file:///Users/roly/').on('error', (err) => {
-          assert.ok(err);
-          done();
-        });
+  describe('with no URL', () => {
+    it('Emits error', (done) => {
+      miniget(undefined).on('error', (err) => {
+        assert.ok(err);
+        assert.equal(err.message, 'Invalid URL: undefined');
+        done();
       });
     });
   });
@@ -221,7 +188,9 @@ describe('Make a request', () => {
           .reply(302, '', { Location: 'http://yoursite.com/redirect-2' })
           .get('/redirect-2')
           .reply(302, '', { Location: 'http://yoursite.com/redirect-3' });
-        const stream = miniget('http://yoursite.com/first-request');
+        const stream = miniget('http://yoursite.com/first-request', {
+          maxRedirects: 2,
+        });
         stream.on('error', (err) => {
           assert.ok(err);
           scope.done();
@@ -276,6 +245,26 @@ describe('Make a request', () => {
         clock.tick(1000);
       });
     });
+    it('Emits error after multiple tries', (done) => {
+      const scope = nock('https://mysite.io')
+        .get('/api/v1/data')
+        .reply(429, 'too many requests')
+        .get('/api/v1/data')
+        .reply(429, 'too many requests')
+        .get('/api/v1/data')
+        .reply(429, 'too many requests');
+
+      const stream = miniget('https://mysite.io/api/v1/data');
+      stream.on('error', (err) => {
+        assert.ok(err);
+        scope.done();
+        assert.equal(err.message, 'Status code: 429');
+        done();
+      });
+      stream.on('retry', () => {
+        clock.tick(1000);
+      });
+    });
     describe('with `retry-after` header', () => {
       it('Retries after given time', (done) => {
         const scope = nock('https://mysite.io')
@@ -301,40 +290,71 @@ describe('Make a request', () => {
   });
 
   describe('using the `transform` option', () => {
-    it('Calls `transform` function and customizes request', (done) => {
+    it('Calls `transform` function and customizes request', async () => {
       const scope = nock('http://other.com')
         .get('/http://supplies.com/boxes')
         .reply(200, '[  ]');
-      miniget('http://supplies.com/boxes', {
+      let transformCalled = false;
+      let body = await miniget('http://supplies.com/boxes', {
         transform: (parsed) => {
+          transformCalled = true;
           return {
             host: 'other.com',
             path: `/${parsed.protocol}//${parsed.host}${parsed.path}`,
           };
         },
-      }, (err, res, body) => {
-        scope.done();
-        assert.ifError(err);
-        assert.equal(res.statusCode, 200);
-        assert.equal(body, '[  ]');
-        done();
-      });
+      }).text();
+      assert.ok(transformCalled);
+      scope.done();
+      assert.equal(body, '[  ]');
     });
-    it('Calls `transform` function and customizes request with protocol changing', (done) => {
+    it('Calls `transform` function and customizes request with protocol changing', async () => {
       const scope = nock('http://that.com')
         .get('/')
         .reply(200, '[  ]');
-      miniget('https://that.com', {
+      let body = await miniget('https://that.com', {
         transform: (parsed) => {
           parsed.protocol = 'http:';
           return parsed;
         },
-      }, (err, res, body) => {
-        scope.done();
-        assert.ifError(err);
-        assert.equal(res.statusCode, 200);
-        assert.equal(body, '[  ]');
-        done();
+      }).text();
+      scope.done();
+      assert.equal(body, '[  ]');
+    });
+    describe('with bad URL', () => {
+      it('Catches error', (done) => {
+        let stream = miniget('http://supplies.com/boxes', {
+          transform: (parsed) => {
+            parsed.protocol = 'file';
+            return parsed;
+          },
+        });
+        stream.on('error', (err) => {
+          assert.equal(err.message, 'Invalid URL object from `transform` function');
+          done();
+        });
+      });
+    });
+    describe('with no object returned', () => {
+      it('Catches error', (done) => {
+        let stream = miniget('http://supplies.com/boxes', {
+          transform: () => undefined,
+        });
+        stream.on('error', (err) => {
+          assert.equal(err.message, 'Invalid URL object from `transform` function');
+          done();
+        });
+      });
+    });
+    describe('that throws', () => {
+      it('Catches error', (done) => {
+        let stream = miniget('http://kanto.com', {
+          transform: () => { throw Error('hello'); },
+        });
+        stream.on('error', (err) => {
+          assert.equal(err.message, 'hello');
+          done();
+        });
       });
     });
   });
@@ -449,7 +469,8 @@ describe('Make a request', () => {
           });
           let res: IncomingMessage;
           stream.on('response', (a) => {
-            res = a; });
+            res = a;
+          });
           let reconnects = 0;
           stream.on('reconnect', () => {
             reconnects++;
@@ -582,7 +603,7 @@ describe('Make a request', () => {
 
   describe('that gets aborted', () => {
     describe('immediately', () => {
-      it('Does not call callback or end stream', (done) => {
+      it('Does not end stream', (done) => {
         nock('http://anime.me')
           .get('/')
           .reply(200, 'ooooaaaaaaaeeeee');
@@ -635,7 +656,7 @@ describe('Make a request', () => {
       });
     });
 
-    it('Decompresses stream', (done) => {
+    it('Decompresses stream', async () => {
       const res = fs.createReadStream(file).pipe(zlib.createGzip());
       const scope = nock('http://yoursite.com', {
         reqheaders: { 'Accept-Encoding': 'gzip' }
@@ -649,16 +670,13 @@ describe('Make a request', () => {
         acceptEncoding: { gzip: (): Transform => zlib.createGunzip() },
         maxRetries: 0,
       });
-      streamEqual(fs.createReadStream(file), stream, (err, equal) => {
-        assert.ifError(err);
-        assert.ok(equal);
-        scope.done();
-        done();
-      });
+      let equal = await streamEqual(fs.createReadStream(file), stream);
+      assert.ok(equal);
+      scope.done();
     });
 
     describe('compressed twice', () => {
-      it('Decompresses stream', (done) => {
+      it('Decompresses stream', async () => {
         const res = fs.createReadStream(file)
           .pipe(zlib.createGzip())
           .pipe(zlib.createDeflate());
@@ -677,17 +695,14 @@ describe('Make a request', () => {
           },
           maxRetries: 0,
         });
-        streamEqual(fs.createReadStream(file), stream, (err, equal) => {
-          assert.ifError(err);
-          assert.ok(equal);
-          scope.done();
-          done();
-        });
+        let equal = await streamEqual(fs.createReadStream(file), stream);
+        assert.ok(equal);
+        scope.done();
       });
     });
 
     describe('compressed incorrectly', () => {
-      it('Emits compression error', (done) => {
+      it('Emits compression error', async () => {
         const res = fs.createReadStream(file)
           .pipe(zlib.createGzip())
           .pipe(zlib.createDeflate());
@@ -706,17 +721,16 @@ describe('Make a request', () => {
             deflate: (): Transform => zlib.createInflate(),
           }
         });
-        streamEqual(fs.createReadStream(file), stream, (err) => {
-          assert.ok(err);
-          assert.equal(err.message, 'incorrect header check');
-          scope.done();
-          done();
-        });
+        await assert.rejects(
+          streamEqual(fs.createReadStream(file), stream),
+          null, 'incorrect header check'
+        );
+        scope.done();
       });
     });
 
     describe('without matching decompressing stream', () => {
-      it('Gets original compressed stream', (done) => {
+      it('Gets original compressed stream', async () => {
         const res = fs.createReadStream(file).pipe(zlib.createGzip());
         const scope = nock('http://yoursite.com', {
           reqheaders: { 'Accept-Encoding': 'deflate' }
@@ -733,12 +747,24 @@ describe('Make a request', () => {
           maxRetries: 0,
         });
         const expected = fs.createReadStream(file).pipe(zlib.createGzip());
-        streamEqual(expected, stream, (err, equal) => {
-          assert.ifError(err);
-          assert.ok(equal);
-          scope.done();
-          done();
-        });
+        let equal = await streamEqual(expected, stream);
+        assert.ok(equal);
+        scope.done();
+      });
+    });
+  });
+
+  describe('with `method = "HEAD"`', () => {
+    it('Emits `response`', (done) => {
+      const scope = nock('http://hello.net')
+        .head('/world')
+        .reply(200, '', { 'content-length': '10' });
+      const req = miniget('http://hello.net/world', { method: 'HEAD' });
+      req.on('error', done);
+      req.on('response', res => {
+        scope.done();
+        assert.equal(res.headers['content-length'], '10');
+        done();
       });
     });
   });
